
import com.sun.opengl.util.Animator;
import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureIO;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCanvas;
import javax.media.opengl.GLEventListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.nio.IntBuffer;
import java.util.Scanner;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;

/**
 * @author Jeremy Smith
 */
public class CS752_Screen extends GLCanvas implements GLEventListener, KeyListener
{
    private int shaderprogram1, shaderprogram2, shaderprogram3, currentshader;
    private final Animator animator = new Animator(this);
    private Texture normalMap, colorMap, displacementMap;
    private CS752_Rock rock;
    
    private float white[] = {1.0f, 1.0f, 1.0f, 1};
    private float gray[] = {0.7f, 0.7f, 0.7f, 1};
    float lightPosition[] = {10.0f, 10.0f, 10.0f, 1};
    float shininess[] = {10.0f};
    
    public CS752_Screen() 
    {
        addGLEventListener(this);
        addKeyListener(this);
        animator.start();
    }

    public void init(GLAutoDrawable drawable) 
    {
        GL gl = drawable.getGL();

        // Setup GL properties
        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDepthFunc(GL.GL_LESS);
        gl.glShadeModel(GL.GL_SMOOTH);
        gl.glClearColor(0.1f, 0.1f, 0.1f, 1.0f);
        
        //create color map, normal map, and displacement map
        gl.glEnable(GL.GL_TEXTURE_2D);
        normalMap = createTextureMap("CS752_NormalMap.jpg");
        colorMap = createTextureMap("CS752_ColorMap.jpg");
        displacementMap = createTextureMap("CS752_DisplacementMap.jpg");
        
        //enable light 
        gl.glEnable(GL.GL_LIGHTING);
        gl.glEnable(GL.GL_LIGHT0);
       
        gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, white, 0);
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, white, 0);
        gl.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, gray, 0);
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, gray, 0);
        gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, white, 0);
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, white, 0);
        gl.glMaterialfv(GL.GL_FRONT, GL.GL_SHININESS, shininess, 0);
        
        gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, lightPosition, 0);
        
        
        //create shader programs
        try 
        {
            shaderprogram1 = attachShaders(gl,"CS752_VS2.txt","CS752_FS1.txt");
            shaderprogram2 = attachShaders(gl,"CS752_VS1.txt","CS752_FS2.txt");
            shaderprogram3 = attachShaders(gl,"CS752_VS1.txt","CS752_FS1.txt");
        } 
        catch (Exception e) 
        {
            e.printStackTrace();
        }
        
        currentshader = shaderprogram1;
        
        rock = new CS752_Rock();
        
        
        
    }
    
    private Texture createTextureMap(String filepath) 
    {
        Texture tex = null;
        InputStream input = CS752_Screen.class.getResourceAsStream(filepath);
        
        try{
            //load texture
            tex = TextureIO.newTexture(input, true, filepath.substring(filepath.indexOf(".")));
            tex.bind();
        }
        catch(IOException ex)
        {
            System.err.println(ex);
        }
        
        return tex;
    }

    public void display(GLAutoDrawable drawable) 
    {
        
        GL gl = drawable.getGL();

        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
        gl.glUseProgram(currentshader);
        
        //apply normal map
        gl.glActiveTexture(GL.GL_TEXTURE0);
        gl.glUniform1i(gl.glGetUniformLocation(currentshader, "normalMap"),0);
        gl.glBindTexture(GL.GL_TEXTURE_2D, normalMap.getTextureObject());
        
        //apply color map
        gl.glActiveTexture(GL.GL_TEXTURE1);
        gl.glUniform1i(gl.glGetUniformLocation(currentshader, "colorMap"),1);
        gl.glBindTexture(GL.GL_TEXTURE_2D, colorMap.getTextureObject());
        
        //apply displacement map
        gl.glActiveTexture(GL.GL_TEXTURE2);
        gl.glUniform1i(gl.glGetUniformLocation(currentshader, "displacementMap"),2);
        gl.glBindTexture(GL.GL_TEXTURE_2D, displacementMap.getTextureObject());
        
        // draw rock
        gl.glLoadIdentity();
        
        rock.draw(gl);
        
        gl.glFlush();
        
        
    }

    public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) 
    {
        GL gl = drawable.getGL();
        gl.glViewport(0, 0, width, height);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        if (width <= (height * 2))
        {
            gl.glOrtho(-6.0, 6.0, -3.0 * ((float) height * 2) / (float) width, 3.0 * ((float) height * 2) / (float) width, -10.0, 10.0);
        }
        else
        {
            gl.glOrtho(-6.0 * (float) width / ((float) height * 2), 6.0 * (float) width / ((float) height * 2), -3.0, 3.0, -10.0, 10.0);
        }
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();
        gl.glOrtho(0, width, 0, height, -1.0, 1.0);
    }

    public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {}
    

    private int attachShaders(GL gl, String vertexFilename, String fragmentFilename) throws Exception 
    {
        //attach vertex shader
        String[] vertexSource = new String[] {readFile(vertexFilename)};
        int vertexShaderProgram = gl.glCreateShader(GL.GL_VERTEX_SHADER);
        gl.glShaderSource(vertexShaderProgram, 1, vertexSource, null, 0);
        gl.glCompileShader(vertexShaderProgram);

        //attach fragement shader
        String[] fragmentSource = new String[] {readFile(fragmentFilename)};
        int fragmentShaderProgram = gl.glCreateShader(GL.GL_FRAGMENT_SHADER);
        gl.glShaderSource(fragmentShaderProgram, 1, fragmentSource, null, 0);
        gl.glCompileShader(fragmentShaderProgram);

        //link shaders
        int shaderprogram = gl.glCreateProgram();
        gl.glAttachShader(shaderprogram, vertexShaderProgram);
        gl.glAttachShader(shaderprogram, fragmentShaderProgram);
        gl.glLinkProgram(shaderprogram);
        gl.glValidateProgram(shaderprogram);
        IntBuffer intBuffer = IntBuffer.allocate(1);
        gl.glGetProgramiv(shaderprogram, GL.GL_LINK_STATUS,intBuffer);
        if (intBuffer.get(0)!=1)
        {
            gl.glGetProgramiv(shaderprogram, GL.GL_INFO_LOG_LENGTH,intBuffer);
            int size = intBuffer.get(0);
            System.err.println("Program link error: ");
            if (size>0)
            {
                ByteBuffer byteBuffer = ByteBuffer.allocate(size);
                gl.glGetProgramInfoLog(shaderprogram, size, intBuffer, byteBuffer);
                for (byte b:byteBuffer.array())
                {
                    System.err.print((char)b);
                }
            } 
            else 
            {
                System.out.println("Unknown");
            }
            System.exit(1);
        }
        
        return shaderprogram;
    }

    private String readFile(String filename)
    {
        String outputString = "";
        InputStream input = CS752_Screen.class.getResourceAsStream(filename);
        
        Scanner scanner = new Scanner(input);
        outputString = scanner.useDelimiter("\\Z").next();
        
        return outputString;
    }

    public void keyTyped(KeyEvent ke) {}

    public void keyPressed(KeyEvent ke) {}

    public void keyReleased(KeyEvent ke) 
    {
        //cycle through shaders when space bar is released
        if(ke.getKeyChar() == ' ')
        {
            if(currentshader == shaderprogram1)
            {
                currentshader = shaderprogram2;
            }
            else if (currentshader == shaderprogram2)
            {
                currentshader = shaderprogram3;
            }
            else if (currentshader == shaderprogram3)
            {
                currentshader = shaderprogram1;
            }
        }
    }
    
}
